# Generated by Django 5.0.9 on 2024-12-31 09:58

import django.db.models.deletion
import django.db.models.lookups
import uuid
from django.db import migrations, models
from django.db.backends.base.schema import BaseDatabaseSchemaEditor
from django.db.migrations.state import ProjectState

class ConditionalDatabaseOperation(migrations.SeparateDatabaseAndState):
    def __init__(self, operation, condition_sql):
        self.condition_sql = condition_sql
        super().__init__(database_operations=operation, state_operations=operation)

    def database_forwards(
            self, app_label: str, schema_editor: BaseDatabaseSchemaEditor,
            from_state: ProjectState, to_state: ProjectState
    ) -> None:
        super().database_forwards(app_label, schema_editor,
            from_state, to_state)

    def database_backwards(
            self, app_label: str, schema_editor: BaseDatabaseSchemaEditor,
            from_state: ProjectState, to_state: ProjectState
    ) -> None:
        super().database_backwards(app_label, schema_editor,
            from_state, to_state)



class Migration(migrations.Migration):

    initial = True

    dependencies = []

    state_operations = [
        migrations.CreateModel(
            name="Pair",
            fields=[
                (
                    "key",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("is_active", models.BooleanField(default=True)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("slug", models.CharField(max_length=256, unique=True)),
                ("name", models.CharField(max_length=256)),
            ],
            options={
                "ordering": ["-created_at"],
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Exchange",
            fields=[
                (
                    "key",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("is_active", models.BooleanField(default=True)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("name", models.CharField(max_length=256, unique=True)),
            ],
            options={
                "ordering": ["-created_at"],
                "abstract": False,
                "indexes": [
                    models.Index(
                        fields=["-created_at"], name="market_exch_created_88028c_idx"
                    ),
                    models.Index(fields=["key"], name="market_exch_key_080597_idx"),
                ],
            },
        ),
        migrations.CreateModel(
            name="Market",
            fields=[
                (
                    "key",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("is_active", models.BooleanField(default=True)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("name", models.CharField(max_length=256, unique=True)),
            ],
            options={
                "ordering": ["-created_at"],
                "abstract": False,
                "indexes": [
                    models.Index(
                        fields=["-created_at"], name="market_mark_created_cb7f1b_idx"
                    ),
                    models.Index(fields=["key"], name="market_mark_key_3eea8f_idx"),
                ],
            },
        ),
        migrations.CreateModel(
            name="ExchangeMarket",
            fields=[
                (
                    "key",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("is_active", models.BooleanField(default=True)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                (
                    "exchange",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="market.exchange",
                    ),
                ),
                (
                    "market",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="market.market"
                    ),
                ),
            ],
            options={
                "ordering": ["-created_at"],
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Network",
            fields=[
                (
                    "key",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("is_active", models.BooleanField(default=True)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("name", models.CharField(db_index=True, max_length=256, unique=True)),
                ("slug", models.CharField(max_length=256, unique=True)),
            ],
            options={
                "ordering": ["-created_at"],
                "abstract": False,
                "indexes": [
                    models.Index(
                        fields=["-created_at"], name="market_netw_created_0dca90_idx"
                    ),
                    models.Index(fields=["key"], name="market_netw_key_a8122b_idx"),
                ],
            },
        ),
        migrations.CreateModel(
            name="ExchangeMarketPairBinding",
            fields=[
                (
                    "key",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("is_active", models.BooleanField(default=True)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("price_decimals", models.IntegerField()),
                ("quantity_decimals", models.IntegerField()),
                ("price_step_size", models.FloatField()),
                ("quantity_step_size", models.FloatField()),
                ("min_notional", models.FloatField()),
                (
                    "exchange_market",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="pair_bindings",
                        to="market.exchangemarket",
                    ),
                ),
                (
                    "pair",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="exchange_market_bindings",
                        to="market.pair",
                    ),
                ),
            ],
            options={
                "ordering": ["-created_at"],
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Symbol",
            fields=[
                (
                    "key",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("is_active", models.BooleanField(default=True)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("name", models.CharField(max_length=256)),
                ("slug", models.CharField(max_length=256, unique=True)),
            ],
            options={
                "ordering": ["-created_at"],
                "abstract": False,
                "indexes": [
                    models.Index(
                        fields=["-created_at"], name="market_symb_created_2ceb88_idx"
                    ),
                    models.Index(fields=["key"], name="market_symb_key_20d70b_idx"),
                ],
            },
        ),
        migrations.AddField(
            model_name="pair",
            name="base",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="base_pairs",
                to="market.symbol",
            ),
        ),
        migrations.AddField(
            model_name="pair",
            name="quote",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="quote_pairs",
                to="market.symbol",
            ),
        ),
        migrations.CreateModel(
            name="NetworkSymbolBinding",
            fields=[
                (
                    "key",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("is_active", models.BooleanField(default=True)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("decimals", models.IntegerField(default=18)),
                ("step_size", models.FloatField(default=1e-18)),
                (
                    "type",
                    models.CharField(
                        choices=[("COIN", "Native Coin"), ("TOKEN", "Utility Token")],
                        db_index=True,
                        default="TOKEN",
                        max_length=64,
                    ),
                ),
                (
                    "network",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="symbol_bindings",
                        to="market.network",
                    ),
                ),
                (
                    "symbol",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="network_bindings",
                        to="market.symbol",
                    ),
                ),
            ],
            options={
                "ordering": ["-created_at"],
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Asset",
            fields=[
                (
                    "key",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("is_active", models.BooleanField(default=True)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("updated_at", models.DateTimeField(auto_now=True)),
                (
                    "type",
                    models.CharField(
                        choices=[("SYMBOL", "SYMBOL"), ("PAIR", "PAIR")], max_length=32
                    ),
                ),
                (
                    "pair",
                    models.OneToOneField(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="market.pair",
                    ),
                ),
                (
                    "symbol",
                    models.OneToOneField(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="market.symbol",
                    ),
                ),
            ],
            options={
                "ordering": ["-created_at"],
                "abstract": False,
            },
        ),
        migrations.AddIndex(
            model_name="exchangemarket",
            index=models.Index(
                fields=["-created_at"], name="market_exch_created_1197a6_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="exchangemarket",
            index=models.Index(fields=["key"], name="market_exch_key_88a8ff_idx"),
        ),
        migrations.AlterUniqueTogether(
            name="exchangemarket",
            unique_together={("exchange", "market")},
        ),
        migrations.AddIndex(
            model_name="exchangemarketpairbinding",
            index=models.Index(
                fields=["-created_at"], name="market_exch_created_fb3bee_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="exchangemarketpairbinding",
            index=models.Index(fields=["key"], name="market_exch_key_94627e_idx"),
        ),
        migrations.AlterUniqueTogether(
            name="exchangemarketpairbinding",
            unique_together={("exchange_market", "pair")},
        ),
        migrations.AddIndex(
            model_name="pair",
            index=models.Index(
                fields=["-created_at"], name="market_pair_created_da7b72_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="pair",
            index=models.Index(fields=["key"], name="market_pair_key_943411_idx"),
        ),
        migrations.AlterUniqueTogether(
            name="pair",
            unique_together={("base", "quote")},
        ),
        migrations.AddIndex(
            model_name="networksymbolbinding",
            index=models.Index(
                fields=["-created_at"], name="market_netw_created_821044_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="networksymbolbinding",
            index=models.Index(fields=["key"], name="market_netw_key_f0a067_idx"),
        ),
        migrations.AddConstraint(
            model_name="networksymbolbinding",
            constraint=models.UniqueConstraint(
                models.F("network"),
                models.F("symbol"),
                name="network_symbol_binding_unique_network_symbol",
            ),
        ),
        migrations.AddConstraint(
            model_name="networksymbolbinding",
            constraint=models.UniqueConstraint(
                condition=models.Q(("type", "COIN")),
                fields=("network",),
                name="network_symbol_binding_unique_network_native_coin",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="networksymbolbinding",
            unique_together={("symbol", "network")},
        ),
        migrations.AddIndex(
            model_name="asset",
            index=models.Index(
                fields=["-created_at"], name="market_asse_created_afd5ed_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="asset",
            index=models.Index(fields=["key"], name="market_asse_key_04b9f4_idx"),
        ),
        migrations.AddConstraint(
            model_name="asset",
            constraint=models.CheckConstraint(
                check=models.ExpressionWrapper(
                    models.Q(
                        django.db.models.lookups.Exact(
                            lhs=models.Func(
                                "symbol",
                                "pair",
                                function="num_nonnulls",
                                output_field=models.IntegerField(),
                            ),
                            rhs=models.Value(1),
                        )
                    ),
                    output_field=models.BooleanField(),
                ),
                name="exactly_one_non_null__symbol__pair",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="asset",
            unique_together={("type", "symbol", "pair")},
        ),
    ]

    operations = state_operations
